@@ -19,7 +19,9 @@ Histogram = core.Histogram
 CONTENT_TYPE_LATEST = exposition.CONTENT_TYPE_LATEST
 generate_latest = exposition.generate_latest
 MetricsHandler = exposition.MetricsHandler
+make_wsgi_app = exposition.make_wsgi_app
 start_http_server = exposition.start_http_server
+start_wsgi_server = exposition.start_wsgi_server
 write_to_textfile = exposition.write_to_textfile
 push_to_gateway = exposition.push_to_gateway
 pushadd_to_gateway = exposition.pushadd_to_gateway
