@@ -15,13 +15,14 @@ try:
     from BaseHTTPServer import HTTPServer
     from urllib2 import build_opener, Request, HTTPHandler
     from urllib import quote_plus
+    from urlparse import parse_qs, urlparse
 except ImportError:
     # Python 3
     unicode = str
     from http.server import BaseHTTPRequestHandler
     from http.server import HTTPServer
     from urllib.request import build_opener, Request, HTTPHandler
-    from urllib.parse import quote_plus
+    from urllib.parse import quote_plus, parse_qs, urlparse
 
 
 CONTENT_TYPE_LATEST = str('text/plain; version=0.0.4; charset=utf-8')
@@ -31,10 +32,16 @@ CONTENT_TYPE_LATEST = str('text/plain; version=0.0.4; charset=utf-8')
 def make_wsgi_app(registry=core.REGISTRY):
     '''Create a WSGI app which serves the metrics from a registry.'''
     def prometheus_app(environ, start_response):
+        params = parse_qs(environ['QUERY_STRING'])
+        r = registry
+        if 'name[]' in params:
+            r = r.restricted_registry(params['name[]'])
+        output = generate_latest(r)
+
         status = str('200 OK')
         headers = [(str('Content-type'), CONTENT_TYPE_LATEST)]
         start_response(status, headers)
-        return [generate_latest(registry)]
+        return [output]
     return prometheus_app
 
 
@@ -70,10 +77,19 @@ def generate_latest(registry=core.REGISTRY):
 
 class MetricsHandler(BaseHTTPRequestHandler):
     def do_GET(self):
+        registry = core.REGISTRY
+        params = parse_qs(urlparse(self.path).query)
+        if 'name[]' in params:
+            registry = registry.restricted_registry(params['name[]'])
+        try:
+            output = generate_latest(registry)
+        except:
+            self.send_error(500, 'error generating metric output')
+            raise
         self.send_response(200)
         self.send_header('Content-Type', CONTENT_TYPE_LATEST)
         self.end_headers()
-        self.wfile.write(generate_latest(core.REGISTRY))
+        self.wfile.write(output)
 
     def log_message(self, format, *args):
         return
@@ -105,6 +121,16 @@ def write_to_textfile(path, registry):
 def push_to_gateway(gateway, job, registry, grouping_key=None, timeout=None):
     '''Push metrics to the given pushgateway.
 
+    `gateway` the url for your push gateway. Either of the form
+              'http://pushgateway.local', or 'pushgateway.local'.
+              Scheme defaults to 'http' if none is provided
+    `job` is the job label to be attached to all pushed metrics
+    `registry` is an instance of CollectorRegistry
+    `grouping_key` please see the pushgateway documentation for details.
+                   Defaults to None
+    `timeout` is how long push will attempt to connect before giving up.
+              Defaults to None
+
     This overwrites all metrics with the same job and grouping_key.
     This uses the PUT HTTP method.'''
     _use_gateway('PUT', gateway, job, registry, grouping_key, timeout)
@@ -113,6 +139,16 @@ def push_to_gateway(gateway, job, registry, grouping_key=None, timeout=None):
 def pushadd_to_gateway(gateway, job, registry, grouping_key=None, timeout=None):
     '''PushAdd metrics to the given pushgateway.
 
+    `gateway` the url for your push gateway. Either of the form
+              'http://pushgateway.local', or 'pushgateway.local'.
+              Scheme defaults to 'http' if none is provided
+    `job` is the job label to be attached to all pushed metrics
+    `registry` is an instance of CollectorRegistry
+    `grouping_key` please see the pushgateway documentation for details.
+                   Defaults to None
+    `timeout` is how long push will attempt to connect before giving up.
+              Defaults to None
+
     This replaces metrics with the same name, job and grouping_key.
     This uses the POST HTTP method.'''
     _use_gateway('POST', gateway, job, registry, grouping_key, timeout)
@@ -121,13 +157,24 @@ def pushadd_to_gateway(gateway, job, registry, grouping_key=None, timeout=None):
 def delete_from_gateway(gateway, job, grouping_key=None, timeout=None):
     '''Delete metrics from the given pushgateway.
 
+    `gateway` the url for your push gateway. Either of the form
+              'http://pushgateway.local', or 'pushgateway.local'.
+              Scheme defaults to 'http' if none is provided
+    `job` is the job label to be attached to all pushed metrics
+    `grouping_key` please see the pushgateway documentation for details.
+                   Defaults to None
+    `timeout` is how long delete will attempt to connect before giving up.
+              Defaults to None
+
     This deletes metrics with the given job and grouping_key.
     This uses the DELETE HTTP method.'''
     _use_gateway('DELETE', gateway, job, None, grouping_key, timeout)
 
 
 def _use_gateway(method, gateway, job, registry, grouping_key, timeout):
-    url = 'http://{0}/metrics/job/{1}'.format(gateway, quote_plus(job))
+    if not (gateway.startswith('http://') or gateway.startswith('https://')):
+        gateway = 'http://{0}'.format(gateway)
+    url = '{0}/metrics/job/{1}'.format(gateway, quote_plus(job))
 
     data = b''
     if method != 'DELETE':
